function isNode() {
	if (typeof window !== "undefined") {
		return false
	}

	if (typeof process === "undefined") {
		return false
	}

	return true
}

function getEnvironment() {
	return isNode() ? "node" : "browser"
}

async function importDependenciesForEnvironment(environment) {
	if (environment !== "node") return {}

	let node_modules = {};

	const dependencies = ["fs", "os", "path", "crypto", "process"];

	for (const dependency of dependencies) {
		const {default: node_module} = await import(`node:${dependency}`);

		node_modules[dependency] = node_module;
	}

	return node_modules
}

function getInitialStateForEnvironment(environment) {
	if (environment !== "node") return {}

	return {
		cleanupHandler_set: false,
		cleanup_items: []
	}
}

function getGlobalDataObjectForEnvironment(environment) {
	const global_data_symbol = Symbol.for(`@anio-js-foundation/create-temporary-resource-factory-${environment}`);

	if (!(global_data_symbol in globalThis)) {
		globalThis[global_data_symbol] = getInitialStateForEnvironment(environment);
	}

	return globalThis[global_data_symbol]
}

function node_cleanupHandler(dependencies, global_vars) {
	const {fs} = dependencies;

	while (global_vars.cleanup_items.length) {
		const item = global_vars.cleanup_items.shift();

		try {
			fs.unlinkSync(item);
		} catch {}
	}
}

function node_createResource({dependencies, global_vars}, data, options) {
	const {fs, os, path, crypto, process} = dependencies;

	if (!global_vars.cleanupHandler_set) {
		process.on("SIGTERM", () => {
			node_cleanupHandler(dependencies, global_vars);
		});

		process.on("exit", () => {
			node_cleanupHandler(dependencies, global_vars);
		});

		global_vars.cleanupHandler_set = true;
	}

	const tmpname = crypto.randomBytes(16).toString("hex");
	let extension = ".txt";

	if (options.type === "text/javascript") {
		extension = ".mjs";
	}

	const tmppath = path.join(os.tmpdir(), tmpname + extension);

	// make sure file is created exclusively and with only
	// owner having write permission
	const fd = fs.openSync(tmppath, "wx+", 0o644);

	fs.writeSync(fd, data);
	fs.closeSync(fd);

	// protect file by setting it to read only
	fs.chmodSync(tmppath, 0o444);

	const location = fs.realpathSync(tmppath);

	if (options.auto_cleanup) {
		global_vars.cleanup_items.push(location);
	}

	return {
		location,
		cleanup() {
			try {
				fs.unlinkSync(location);
			} catch {}
		}
	}
}

const environment = getEnvironment();
const global_vars = getGlobalDataObjectForEnvironment(environment);
const dependencies = await importDependenciesForEnvironment(environment);
const createResource = environment === "node" ? node_createResource : () => {};

function createTemporaryResource(data, {type = "text/plain", auto_cleanup = true} = {}) {
	return createResource({dependencies, global_vars}, data, {
		type,
		auto_cleanup
	})
}

export { createTemporaryResource as default };
